#if !defined( PROCMON_INFO_H )
#define PROCMON_INFO_H
#include <stdio.h>
#define PROCMON_SAMPLE_SAVE 2
#define PROCMON_MAX_LEN 1024
/* procmon_info_get return: likely never fill with data (eg process gone) */
#define PROCMON_ERROR 1
/* procmon_info_get return: current sampling suspect and pinfo not filled */
#define PROCMON_WARNING 2
/* ==========================================================================
 * Copy:
 *    procmon_info_copy( PROCMON_INFO_struct *pinfo_orig,
 *                       PROCMON_INFO_struct *pinfo_copy )
 * Fill:
 *    procmon_info_get( long long pid,
 *                      PROCMON_INFO_struct *pinfo )
 * Init:
 *    procmon_info_init( PROCMON_INFO_struct *pinfo )
 *
 * Print:
 *    procmon_info_print( PROCMON_INFO_struct *pinfo,
 *                        PROCMON_INFO_SETTINGS_struct *psettings,
 *                        FILE *outfile )
 *
 * Usage:
 * ------
 * Link in libprocmon_info.a
 * C code sample:
 *    #include "procmon_info.h"
 *    ...<in some routine>...
 *    PROCMON_INFO_struct pinfo
 *    procmon_info_init( &pinfo ); // initialize
 *    procmon_info_get( 0, &pinfo ); // get info about this process
 *    procmon_info_print( &pinfo, NULL, NULL ); // print pinfo struct to stdout
 *    ...<some more code>...
 *    procmon_info_get( 0, &pinfo ); // get another sampling (use same pinfo)
 *    procmon_info_print( &pinfo, NULL, NULL ); // print pinfo
 *
 * Notes
 * -----
 * o Units and Datatypes
 *   Times are in seconds.  Sizes are in bytes.
 *   Use simple data types (double, long long)
 *    - Simpler printing of values as doubles.
 *    - Easier future(?) translation to Fortran.
 *     (no unsigned long long or variable sized long)
 *    - Struct only contains 8-byte members to avoid padding issues when
 *      mixing compilers.
 *    - EpochTime will still have about 6 digits of seconds precision
 *      (usecs) - which is relly the same precision as the clock I am using
 *      anyway.
 * o Specialized Use
 *   Users might wish to call procmon_info_get() at certain points in
 *   their code to get information at interesting times or to gracefully
 *   terminate if, for example, the amount of free machine memory drops
 *   below 10%.
 * ======================================================================== */
typedef struct PROCMON_INFO_struct
{
  long long num_samples; /* number of samplings pinfo has successfully done */
  long long printed_header; /* if already printed the header
                             * Set to !0 if never want to print header */
  long long chars_printed; /* the number of characters printed */
  double wall;        /* secs EpochTime - for getting common relative time */
  /* machine (m) info */
  double m_size;      /* total memory in bytes */
  double m_free;      /* free memory in bytes */
  /* static process (p) info */
  long long p_pid;    /* process id */
  long long p_ppid;   /* parent process id */
  long long p_uid;    /* user ID of process */
  char p_command[PROCMON_MAX_LEN]; /* process command */
  long long thread;         /* if this is a thread or not (not implemented) */
  double p_size_lim;  /* byte size limit (<=0 if unknown) */
  double p_rss_lim;   /* byte rss size limit (<=0 if unknown) */
  double p_data_lim;  /* byte data size limit (<=0 if unknown) */
  double p_stack_lim; /* byte stack size limit (<=0 if unknown) */
  /* dynamic process (p) info */
  double p_size[PROCMON_SAMPLE_SAVE]; /* byte process size [cur, max] */
  double p_rss;       /* byte process resident set size */
  double p_data;      /* byte process data size */
  double p_stack;     /* byte process stack size */
  double p_time[PROCMON_SAMPLE_SAVE]; /* sec proc time [current, previous] */
  double p_wall[PROCMON_SAMPLE_SAVE]; /* sec proc wall clock time since this
                                         process started: [cur, previous] */
  double p_majflt;    /* number of major page faults - disk reads */
  double p_minflt;    /* number of minor page faults - no disk reads */
}
PROCMON_INFO_struct;
/* ==========================================================================
 * Fill:
 *    procmon_info_settings_get( int argc,
 *                               const char **argv,
 *                               PROCMON_INFO_SETTINGS_struct *psettings )
 *
 * Print:
 *    procmon_info_settings_print( int argc,
 *                                 const char **argv,
 *                                 PROCMON_INFO_SETTINGS_struct *psettings )
 *
 * Usage:
 * ------
 * Link in libprocmon_info.a
 * C code sample:
 *    #include "procmon_info.h"
 *    ...<in some routine>...
 *    PROCMON_INFO_SETTINGS_struct psettings;
 *    procmon_info_settings_get( &psettings ); // fill settings struct
 *    procmon_info_settings_print( NULL, &psettings ); // print struct - stdout
 *
 * Notes
 * -----
 * This routine is used by the procmon internals to get environment
 * variable settings (and print them in the header of the output).  It
 * is also used by the standalone executable procmon to parse the command
 * line options.  It might be useful to other users as well (eg for getting
 * env var settings and creating outfile_name).
 * ======================================================================== */
typedef struct PROCMON_INFO_SETTINGS_struct
{
  /* ---------------------------------- */
  /* env vars:          PROCMON_<NAME>  */
  /* command line args: -<name> <value> */
  /* ---------------------------------- */
  long long
    time,                 /* time (milliseconds) between samplings */
    interval,             /* print every n-th sample  */
    min;                  /* minimum number of samples to keep file */
  char
    dir[PROCMON_MAX_LEN]; /* directory setting */
  /* ------------ */
  /* other things */
  /* ------------ */
  long long
    pid,                  /* process id to get info about (0) */
    pretty,               /* whether or not to try and do pretty output (0) */
    prettyw,              /* screen width (used when cannot get width) */
    uid;                  /* process uid to get info about (-1) */
  char
    command[PROCMON_MAX_LEN], /* command to get info about
                               * strlen will be 0 if not set */
    outfile_name[PROCMON_MAX_LEN], /* outfile_prefix + <pid>.txt
                                    * The pid part of the name is assumed to
                                    * be the pid of the process (or the pid
                                    * specified on the command line).  If
                                    * you are using command line arguments
                                    * that specify UID/command, this name will
                                    * be incorrect and you should make your
                                    * own name.
                                    * strlen will be 0 for stdout. */
    outfile_prefix[PROCMON_MAX_LEN]; /* output file name prefix:
                                      *   <dir>/procmon_<hostname>_
                                      * Full file name will have suffix:
                                      *   <pid>.txt
                                      * strlen will be 0 for stdout */
}
PROCMON_INFO_SETTINGS_struct;
/* ======================================================================= */
/* ============================== PROTOTYPES ============================= */
/* ======================================================================= */
/*
 * procmon_info_copy - create copy of pinfo struct
 * pinfo:  original
 * pinfoc: copy
 */
int procmon_info_copy(
                      const PROCMON_INFO_struct *pinfo,
                      PROCMON_INFO_struct *pinfoc
                      );
/*
 * procmon_info_get - fills pinfo struct (call procmon_info_init first)
 * pid:   process ID (use 0 for current process)
 * pinfo: process info struct to fill
 */
int procmon_info_get(
                      long long pid,
                      PROCMON_INFO_struct *pinfo
                      );
/*
 * procmon_info_init - initializes pinfo struct
 * pinfo: process info struct
 */
int procmon_info_init(
                      PROCMON_INFO_struct *pinfo
                      );
/*
 * procmon_info_print - print a pinfo struct
 * pinfo:   process info struct
 * outfile: file pointer (if NULL, stdout will be used)
 */
int procmon_info_print(
                       PROCMON_INFO_struct *pinfo,
                       PROCMON_INFO_SETTINGS_struct *psettings,
                       FILE *outfile
                       );
/*
 * procmon_info_settings_get - get env and command line settings
 * argc:           from main( argc, argv ) - if 0,    only uses env vars
 * argv:           from main( argc, argv ) - if NULL, only uses env vars
 * psettings: struct to fill
 */
int procmon_info_settings_get(
                              int argc,
                              const char **argv,
                              PROCMON_INFO_SETTINGS_struct *psettings
                              );
/*
 * procmon_info_settings_print - print settings struct
 * argc:           from main( argc, argv ) - if 0,    only uses env vars
 * argv:           from main( argc, argv ) - if NULL, only uses env vars
 * psettings: struct to fill
 */
int procmon_info_settings_print(
                                const PROCMON_INFO_SETTINGS_struct *psettings,
                                FILE *outfile
                                );
#endif /* PROCMON_INFO_H */
